@@ -34,10 +34,6 @@
 #include <stdio.h>
 #include <string.h>
 
-static const char *__tooltip_help =
-"When ToolTips are enabled, descriptions will be displayed next to options "
-"when the mouse is held over them.";
-
 static const char *__status_bar_help =
 "The status bar in the bottom "
 "left of the nvidia-settings GUI displays the most "
@@ -55,7 +51,7 @@ static const char *__x_display_names_help =
 "When the current settings are saved to the "
 "configuration file, the attributes can either be "
 "qualified with just the screen to which the attribute "
-"should be applied, or the attribute can be qualifed with "
+"should be applied, or the attribute can be qualified with "
 "the entire X Display name.  If you want to be able to "
 "use the same configuration file across multiple "
 "computers, be sure to leave this option unchecked.  "
@@ -81,7 +77,6 @@ static const char *__update_rules_on_profile_name_change_help =
 static void ctk_config_class_init(CtkConfigClass *ctk_config_class);
 
 static void display_status_bar_toggled(GtkWidget *, gpointer);
-static void tooltips_toggled(GtkWidget *, gpointer);
 static void slider_text_entries_toggled(GtkWidget *, gpointer);
 static void display_name_toggled(GtkWidget *widget, gpointer user_data);
 static void show_quit_dialog_toggled(GtkWidget *widget, gpointer user_data);
@@ -138,7 +133,19 @@ static void ctk_config_class_init(CtkConfigClass *ctk_config_class)
                               G_TYPE_NONE, 0);
 }
 
-GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
+void ctk_statusbar_init(CtkStatusBar *status_bar)
+{
+
+    status_bar->widget = gtk_statusbar_new();
+    status_bar->prev_message_id = 0;
+    status_bar->enabled = TRUE;
+#ifndef CTK_GTK3
+    gtk_statusbar_set_has_resize_grip
+        (GTK_STATUSBAR(status_bar->widget), FALSE);
+#endif
+}
+
+GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlSystem *pCtrlSystem)
 {
     gint i;
     GObject *object;
@@ -159,12 +166,6 @@ GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
         const char *help_text;
     } config_check_button_entries[] =
     {
-        { 
-           "Enable ToolTips",
-           CONFIG_PROPERTIES_TOOLTIPS,
-           G_CALLBACK(tooltips_toggled),
-           __tooltip_help
-         },
         {
             "Display Status Bar",
             CONFIG_PROPERTIES_DISPLAY_STATUS_BAR,
@@ -203,28 +204,19 @@ GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
     ctk_config = CTK_CONFIG(object);
 
     ctk_config->conf = conf;
-    ctk_config->pCtrlHandles = pCtrlHandles;
+    ctk_config->pCtrlSystem = pCtrlSystem;
 
     gtk_box_set_spacing(GTK_BOX(ctk_config), 10);
     
+
     /* initialize the statusbar widget */
+    ctk_statusbar_init(&ctk_config->status_bar);
 
-    ctk_config->status_bar.widget = gtk_statusbar_new();
-    ctk_config->status_bar.prev_message_id = 0;
-    ctk_config->status_bar.enabled = TRUE;
-    
-    gtk_statusbar_set_has_resize_grip
-        (GTK_STATUSBAR(ctk_config->status_bar.widget), FALSE);
-    
-    /* XXX force the status bar window to be vertially centered */
-
-    gtk_misc_set_alignment
-        (GTK_MISC(GTK_STATUSBAR(ctk_config->status_bar.widget)->label),
-         0.0, 0.5);
-    
+#ifndef CTK_GTK3
     /* initialize the tooltips widget */
 
     ctk_config->tooltips.object = gtk_tooltips_new();
+#endif
 
     /* banner */
 
@@ -257,13 +249,7 @@ GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
         gtk_container_add(GTK_CONTAINER(check_button), label);
 
         b = !!(ctk_config->conf->booleans & config_check_button_entries[i].mask);
-        if (config_check_button_entries[i].mask == CONFIG_PROPERTIES_TOOLTIPS) {
-            if (b) {
-                gtk_tooltips_enable(ctk_config->tooltips.object);
-            } else {
-                gtk_tooltips_disable(ctk_config->tooltips.object);
-            }
-        }
+
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), b);
         gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
         g_signal_connect(G_OBJECT(check_button), "toggled",
@@ -303,19 +289,15 @@ GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
     gtk_box_pack_start(GTK_BOX(ctk_config), alignment, TRUE, TRUE, 0);
 
     /* Create the file selector for rc file */
-    ctk_config->rc_file_selector =
-        gtk_file_selection_new ("Please select a file to save to");
-
     g_signal_connect(G_OBJECT(ctk_config->button_save_rc), "clicked",
                      G_CALLBACK(save_rc_clicked),
                      (gpointer) ctk_config);
 
-    gtk_file_selection_set_filename
-        (GTK_FILE_SELECTION(ctk_config->rc_file_selector), DEFAULT_RC_FILE);
-
     ctk_config_set_tooltip(ctk_config, ctk_config->button_save_rc,
                            __save_current_config_help);
 
+    ctk_config->rc_filename = NULL;
+
     gtk_widget_show_all(GTK_WIDGET(ctk_config));
 
     return GTK_WIDGET(ctk_config);
@@ -328,61 +310,86 @@ GtkWidget* ctk_config_new(ConfigProperties *conf, CtrlHandles *pCtrlHandles)
 
 static void save_rc_clicked(GtkWidget *widget, gpointer user_data)
 {
-    gint result;
-    const gchar *rc_filename = NULL;
+    gchar *filename = NULL;
     CtkConfig *ctk_config = CTK_CONFIG(user_data);
     CtkWindow *ctk_window =
         CTK_WINDOW(ctk_get_parent_window(GTK_WIDGET(ctk_config)));
 
-    result = gtk_dialog_run(GTK_DIALOG(ctk_config->rc_file_selector));
-    gtk_widget_hide(ctk_config->rc_file_selector);
+    filename =
+        ctk_get_filename_from_dialog("Please select a file to save to.",
+                                     GTK_WINDOW(ctk_window),
+                                     ctk_config->rc_filename ?
+                                         ctk_config->rc_filename :
+                                         DEFAULT_RC_FILE);
 
-    switch (result) {
-    case GTK_RESPONSE_ACCEPT:
-    case GTK_RESPONSE_OK:
-        rc_filename = gtk_file_selection_get_filename
-                          (GTK_FILE_SELECTION(ctk_config->rc_file_selector));
-        break;
-    default:
+    if (!filename) {
         return;
     }
 
+    g_free(ctk_config->rc_filename);
+    ctk_config->rc_filename = filename;
+
     /* write the configuration file */
     add_special_config_file_attributes(ctk_window);
-    nv_write_config_file(rc_filename, ctk_config->pCtrlHandles,
+    nv_write_config_file(ctk_config->rc_filename, ctk_config->pCtrlSystem,
                          ctk_window->attribute_list, ctk_config->conf);
 }
 
+void ctk_statusbar_clear(CtkStatusBar *status_bar)
+{
+    if ((!status_bar->enabled) ||
+        (!status_bar->widget)) {
+        return;
+    }
 
-void ctk_config_statusbar_message(CtkConfig *ctk_config, const char *fmt, ...)
+    if (status_bar->prev_message_id) {
+        gtk_statusbar_remove(GTK_STATUSBAR(status_bar->widget),
+                             1, status_bar->prev_message_id);
+    }
+}
+
+void ctk_statusbar_message(CtkStatusBar *status_bar,
+                           const gchar *str)
+{
+
+    if ((!status_bar->enabled) ||
+        (!status_bar->widget)) {
+        return;
+    }
+
+    if (status_bar->prev_message_id) {
+        gtk_statusbar_remove(GTK_STATUSBAR(status_bar->widget),
+                             1, status_bar->prev_message_id);
+    }
+
+    status_bar->prev_message_id =
+        gtk_statusbar_push
+        (GTK_STATUSBAR(status_bar->widget), 1, str);
+
+} /* ctk_config_statusbar_message() */
+
+void ctk_config_statusbar_message(CtkConfig *ctk_config,
+                                  const char *fmt,
+                                  ...)
 {
     va_list ap;
     gchar *str;
 
     if ((!ctk_config) ||
-        (!ctk_config->status_bar.enabled) ||
-        (!ctk_config->status_bar.widget) ||
         (!(ctk_config->conf->booleans &
            CONFIG_PROPERTIES_DISPLAY_STATUS_BAR))) {
         return;
     }
 
-    if (ctk_config->status_bar.prev_message_id) {
-        gtk_statusbar_remove(GTK_STATUSBAR(ctk_config->status_bar.widget),
-                             1, ctk_config->status_bar.prev_message_id);
-    }
-
     va_start(ap, fmt);
     str = g_strdup_vprintf(fmt, ap);
     va_end(ap);
-    
-    ctk_config->status_bar.prev_message_id =
-        gtk_statusbar_push
-        (GTK_STATUSBAR(ctk_config->status_bar.widget), 1, str);
+
+    ctk_statusbar_message(&ctk_config->status_bar,
+                          str);
 
     g_free(str);
-    
-} /* ctk_config_statusbar_message() */
+}
 
 
 
@@ -398,7 +405,11 @@ void ctk_config_set_tooltip(CtkConfig *ctk_config,
                             const gchar *text)
 
 {
+#ifdef CTK_GTK3
+    gtk_widget_set_tooltip_text(widget, text);
+#else
     gtk_tooltips_set_tip(ctk_config->tooltips.object, widget, text, NULL);
+#endif
 }
 
 
@@ -427,23 +438,6 @@ static void display_status_bar_toggled(
     }
 }
 
-static void tooltips_toggled(GtkWidget *widget, gpointer user_data)
-{
-    CtkConfig *ctk_config = CTK_CONFIG(user_data);
-    gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-
-    if (active) {
-        gtk_tooltips_enable(ctk_config->tooltips.object);
-        ctk_config->conf->booleans |= CONFIG_PROPERTIES_TOOLTIPS;
-    } else {
-        gtk_tooltips_disable(ctk_config->tooltips.object);
-        ctk_config->conf->booleans &= ~CONFIG_PROPERTIES_TOOLTIPS;
-    }
-
-    ctk_config_statusbar_message(ctk_config, "Tooltips %s.", 
-                                 active ? "enabled" : "disabled");
-}
-
 
 static void slider_text_entries_toggled(GtkWidget *widget, gpointer user_data)
 {
@@ -710,6 +704,10 @@ static void enabled_renderer_func(GtkTreeViewColumn *tree_column,
     column = GPOINTER_TO_INT(data);
     gtk_tree_model_get(model, iter, column, &timer_config, -1);
 
+    if (timer_config == NULL) {
+        return;
+    }
+
     value = timer_config->user_enabled;
     g_object_set(GTK_CELL_RENDERER(cell), "active", value, NULL);
 }
@@ -727,6 +725,10 @@ static void description_renderer_func(GtkTreeViewColumn *tree_column,
     column = GPOINTER_TO_INT(data);
     gtk_tree_model_get(model, iter, column, &timer_config, -1);
 
+    if (timer_config == NULL) {
+        return;
+    }
+
     value = timer_config->description;
     g_object_set(GTK_CELL_RENDERER(cell), "text", value, NULL);
 }
@@ -743,6 +745,11 @@ static void time_interval_renderer_func(GtkTreeViewColumn *tree_column,
     gchar str[32];
     
     gtk_tree_model_get(model, iter, column, &timer_config, -1);
+
+    if (timer_config == NULL) {
+        return;
+    }
+
     value = timer_config->interval;
 
     snprintf(str, 32, "%d ms", value);
