@@ -42,10 +42,18 @@ static const char *__ecc_status_help =
 "Returns the current hardware ECC setting "
 "for the targeted GPU.";
 
+static const char *__sbit_error_help =
+"Returns the number of single-bit ECC errors detected by "
+"the targeted GPU since the last system reboot.";
+
 static const char *__dbit_error_help =
 "Returns the number of double-bit ECC errors detected by "
 "the targeted GPU since the last system reboot.";
 
+static const char *__aggregate_sbit_error_help =
+"Returns the number of single-bit ECC errors detected by the "
+"targeted GPU since the last counter reset.";
+
 static const char *__aggregate_dbit_error_help =
 "Returns the number of double-bit ECC errors detected by the "
 "targeted GPU since the last counter reset.";
@@ -67,7 +75,7 @@ static const char *__reset_default_config_button_help =
 static void ecc_config_button_toggled(GtkWidget *, gpointer);
 static void show_ecc_toggle_warning_dlg(CtkEcc *);
 static void ecc_set_config_status(CtkEcc *);
-static void ecc_configuration_update_received(GtkObject *, gpointer, gpointer);
+static void ecc_configuration_update_received(GObject *, CtrlEvent *, gpointer);
 static void post_ecc_configuration_update(CtkEcc *);
 
 GType ctk_ecc_get_type(void)
@@ -154,30 +162,51 @@ static GtkWidget *add_table_int_row(CtkConfig *ctk_config, GtkWidget *table,
 static gboolean update_ecc_info(gpointer user_data)
 {
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
+    CtrlTarget *ctrl_target = ctk_ecc->ctrl_target;
     int64_t val;
     ReturnStatus ret;
 
-    if ( ctk_ecc->ecc_enabled == FALSE ) {
+    if (ctk_ecc->ecc_enabled == FALSE) {
         return FALSE;
     }
-    
+
     /* Query ECC Errors */
 
-    if ( ctk_ecc->dbit_error ) {
-        ret = NvCtrlGetAttribute64(ctk_ecc->handle,
+    if (ctk_ecc->sbit_error) {
+        ret = NvCtrlGetAttribute64(ctrl_target,
+                                   NV_CTRL_GPU_ECC_SINGLE_BIT_ERRORS,
+                                   &val);
+        if (ret != NvCtrlSuccess) {
+            val = 0;
+        }
+        set_label_value(ctk_ecc->sbit_error, val);
+    }
+
+    if (ctk_ecc->dbit_error) {
+        ret = NvCtrlGetAttribute64(ctrl_target,
                                    NV_CTRL_GPU_ECC_DOUBLE_BIT_ERRORS,
                                    &val);
-        if ( ret != NvCtrlSuccess ) {
+        if (ret != NvCtrlSuccess) {
             val = 0;
         }
         set_label_value(ctk_ecc->dbit_error, val);
     }
 
-    if ( ctk_ecc->aggregate_dbit_error ) {
-        ret = NvCtrlGetAttribute64(ctk_ecc->handle,
+    if (ctk_ecc->aggregate_sbit_error) {
+        ret = NvCtrlGetAttribute64(ctrl_target,
+                                   NV_CTRL_GPU_ECC_AGGREGATE_SINGLE_BIT_ERRORS,
+                                   &val);
+        if (ret != NvCtrlSuccess) {
+            val = 0;
+        }
+        set_label_value(ctk_ecc->aggregate_sbit_error, val);
+    }
+
+    if (ctk_ecc->aggregate_dbit_error) {
+        ret = NvCtrlGetAttribute64(ctrl_target,
                                    NV_CTRL_GPU_ECC_AGGREGATE_DOUBLE_BIT_ERRORS,
                                    &val);
-        if ( ret != NvCtrlSuccess ) {
+        if (ret != NvCtrlSuccess) {
             val = 0;
         }
         set_label_value(ctk_ecc->aggregate_dbit_error, val);
@@ -238,13 +267,17 @@ static void ecc_set_config_status(CtkEcc *ctk_ecc)
  * NV-CONTROL client.
  */
 
-static void ecc_configuration_update_received(GtkObject *object,
-                                              gpointer arg1, gpointer user_data)
+static void ecc_configuration_update_received(GObject *object,
+                                              CtrlEvent *event,
+                                              gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
 
-    ctk_ecc->ecc_configured = event_struct->value;
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
+    ctk_ecc->ecc_configured = event->int_attr.value;
  
     /* set ECC configuration button status */
     ecc_set_config_status(ctk_ecc);
@@ -266,9 +299,10 @@ static void reset_default_config_button_clicked(GtkWidget *widget,
     gboolean status;
     ReturnStatus ret;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
-    
+    CtrlTarget *ctrl_target = ctk_ecc->ctrl_target;
+
     /* get default status and set it to ECC configuration */
-    ret = NvCtrlGetAttribute(ctk_ecc->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION,
                              &status);
     if (ret != NvCtrlSuccess) {
@@ -276,7 +310,7 @@ static void reset_default_config_button_clicked(GtkWidget *widget,
                                      "Failed to get default configuration!");
         return;
     }
-    ret =  NvCtrlSetAttribute(ctk_ecc->handle,
+    ret =  NvCtrlSetAttribute(ctrl_target,
                               NV_CTRL_GPU_ECC_CONFIGURATION,
                               status);
     if (ret != NvCtrlSuccess) {
@@ -310,11 +344,12 @@ static void clear_ecc_errors_button_clicked(GtkWidget *widget,
                                             gpointer user_data)
 {
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
-    
-    NvCtrlSetAttribute(ctk_ecc->handle,
+    CtrlTarget *ctrl_target = ctk_ecc->ctrl_target;
+
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_GPU_ECC_RESET_ERROR_STATUS,
                        NV_CTRL_GPU_ECC_RESET_ERROR_STATUS_VOLATILE);
-    
+
     ctk_config_statusbar_message(ctk_ecc->ctk_config,
                                  "ECC errors cleared.");
 } /* clear_ecc_errors_button_clicked() */
@@ -330,11 +365,12 @@ static void clear_aggregate_ecc_errors_button_clicked(GtkWidget *widget,
                                                       gpointer user_data)
 {
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
-    
-    NvCtrlSetAttribute(ctk_ecc->handle,
+    CtrlTarget *ctrl_target = ctk_ecc->ctrl_target;
+
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_GPU_ECC_RESET_ERROR_STATUS,
                        NV_CTRL_GPU_ECC_RESET_ERROR_STATUS_AGGREGATE);
-    
+
     ctk_config_statusbar_message(ctk_ecc->ctk_config,
                                  "ECC aggregate errors cleared.");
 } /* clear_aggregate_ecc_errors_button_clicked() */
@@ -375,6 +411,7 @@ static void ecc_config_button_toggled(GtkWidget *widget,
 {
     gboolean enabled;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
+    CtrlTarget *ctrl_target = ctk_ecc->ctrl_target;
     ReturnStatus ret;
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -383,7 +420,7 @@ static void ecc_config_button_toggled(GtkWidget *widget,
     show_ecc_toggle_warning_dlg(ctk_ecc);
 
     /* set the newly specified ECC value */
-    ret = NvCtrlSetAttribute(ctk_ecc->handle,
+    ret = NvCtrlSetAttribute(ctrl_target,
                              NV_CTRL_GPU_ECC_CONFIGURATION,
                              enabled);
     if (ret != NvCtrlSuccess) {
@@ -391,7 +428,7 @@ static void ecc_config_button_toggled(GtkWidget *widget,
                                      "Failed to set ECC configuration!");
         return;
     }
-    
+
     ctk_ecc->ecc_configured = enabled;
 
     gtk_widget_set_sensitive(ctk_ecc->reset_default_config_button, TRUE);
@@ -402,7 +439,7 @@ static void ecc_config_button_toggled(GtkWidget *widget,
 
 
 
-GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
+GtkWidget* ctk_ecc_new(CtrlTarget *ctrl_target,
                        CtkConfig *ctk_config,
                        CtkEvent *ctk_event)
 {
@@ -410,10 +447,14 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     CtkEcc *ctk_ecc;
     GtkWidget *hbox, *hbox2, *vbox, *hsep, *hseparator, *table;
     GtkWidget *banner, *label, *eventbox;
+    int64_t sbit_error;
+    int64_t aggregate_sbit_error;
     int64_t dbit_error;
     int64_t aggregate_dbit_error;
     gint ecc_config_supported;
     gint val, row = 0;
+    gboolean sbit_error_available;
+    gboolean aggregate_sbit_error_available;
     gboolean dbit_error_available;
     gboolean aggregate_dbit_error_available;
     gboolean ecc_enabled;
@@ -424,13 +465,14 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
 
     /* make sure we have a handle */
 
-    g_return_val_if_fail(handle != NULL, NULL);
+    g_return_val_if_fail((ctrl_target != NULL) &&
+                         (ctrl_target->h != NULL), NULL);
 
     /*
      * check if ECC support available.
      */
 
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_GPU_ECC_SUPPORTED,
                              &val);
     if (ret != NvCtrlSuccess || val != NV_CTRL_GPU_ECC_SUPPORTED_TRUE) {
@@ -442,19 +484,23 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     object = g_object_new(CTK_TYPE_ECC, NULL);
     
     ctk_ecc = CTK_ECC(object);
-    ctk_ecc->handle = handle;
+    ctk_ecc->ctrl_target = ctrl_target;
     ctk_ecc->ctk_config = ctk_config;
     ctk_ecc->ecc_toggle_warning_dlg_shown = FALSE;
 
+    sbit_error_available = TRUE;
     dbit_error_available = TRUE;
+    aggregate_sbit_error_available = TRUE;
     aggregate_dbit_error_available = TRUE;
     
+    sbit_error = 0;
     dbit_error = 0;
+    aggregate_sbit_error = 0;
     aggregate_dbit_error = 0;
 
     /* Query ECC Status */
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_ECC_STATUS,
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_ECC_STATUS,
                              &val);
     if (ret != NvCtrlSuccess || val == NV_CTRL_GPU_ECC_STATUS_DISABLED) {
         ecc_enabled = FALSE;
@@ -467,7 +513,8 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
 
     /* Query ECC Configuration */
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_ECC_CONFIGURATION, &val);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_ECC_CONFIGURATION,
+                             &val);
     if (ret != NvCtrlSuccess ||
         val == NV_CTRL_GPU_ECC_CONFIGURATION_DISABLED) {
             ctk_ecc->ecc_configured = FALSE;
@@ -476,7 +523,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     }
 
     /* get default status */
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION,
                              &val);
     if (ret != NvCtrlSuccess ||
@@ -488,25 +535,40 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
 
     /* Query ECC errors */
     
-    ret = NvCtrlGetAttribute64(handle, NV_CTRL_GPU_ECC_DOUBLE_BIT_ERRORS,
-                             &dbit_error);
-    if ( ret != NvCtrlSuccess ) {
+    ret = NvCtrlGetAttribute64(ctrl_target,
+                               NV_CTRL_GPU_ECC_SINGLE_BIT_ERRORS,
+                               &sbit_error);
+    if (ret != NvCtrlSuccess) {
+        sbit_error_available = FALSE;
+    }
+    ret = NvCtrlGetAttribute64(ctrl_target,
+                               NV_CTRL_GPU_ECC_DOUBLE_BIT_ERRORS,
+                               &dbit_error);
+    if (ret != NvCtrlSuccess) {
         dbit_error_available = FALSE;
     }
-    ret = NvCtrlGetAttribute64(handle,
-                             NV_CTRL_GPU_ECC_AGGREGATE_DOUBLE_BIT_ERRORS,
-                             &aggregate_dbit_error);
-    if ( ret != NvCtrlSuccess ) {
+    ret = NvCtrlGetAttribute64(ctrl_target,
+                               NV_CTRL_GPU_ECC_AGGREGATE_SINGLE_BIT_ERRORS,
+                               &aggregate_sbit_error);
+    if (ret != NvCtrlSuccess) {
+        aggregate_sbit_error_available = FALSE;
+    }
+    ret = NvCtrlGetAttribute64(ctrl_target,
+                               NV_CTRL_GPU_ECC_AGGREGATE_DOUBLE_BIT_ERRORS,
+                               &aggregate_dbit_error);
+    if (ret != NvCtrlSuccess) {
         aggregate_dbit_error_available = FALSE;
     }
+    ctk_ecc->sbit_error_available = sbit_error_available;
+    ctk_ecc->aggregate_sbit_error_available = aggregate_sbit_error_available;
     ctk_ecc->dbit_error_available = dbit_error_available;
     ctk_ecc->aggregate_dbit_error_available = aggregate_dbit_error_available;
     /* Query ECC configuration supported */
 
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_GPU_ECC_CONFIGURATION_SUPPORTED,
                              &ecc_config_supported);
-    if ( ret != NvCtrlSuccess ) {
+    if (ret != NvCtrlSuccess) {
         ecc_config_supported = 0;
     }
 
@@ -560,15 +622,27 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     
     /* Add ECC Errors */
 
-    if ( dbit_error_available ) {
+    if (sbit_error_available && dbit_error_available) {
+        ctk_ecc->sbit_error =
+            add_table_int_row(ctk_config, table, __sbit_error_help,
+                              "Single-bit ECC Errors:", sbit_error,
+                              row, ecc_enabled);
+        row += 1; // add vertical padding between rows
+
         ctk_ecc->dbit_error =
             add_table_int_row(ctk_config, table, __dbit_error_help,
                               "Double-bit ECC Errors:", dbit_error,
                               row, ecc_enabled);
         row += 3; // add vertical padding between rows
     }
-    
-    if ( aggregate_dbit_error_available ) {
+
+    if (aggregate_sbit_error_available && aggregate_dbit_error_available) {
+        ctk_ecc->aggregate_sbit_error =
+            add_table_int_row(ctk_config, table, __aggregate_sbit_error_help,
+                              "Aggregate Single-bit ECC Errors:",
+                              aggregate_sbit_error, row, ecc_enabled);
+        row += 1; // add vertical padding between rows
+
         ctk_ecc->aggregate_dbit_error =
             add_table_int_row(ctk_config, table, __aggregate_dbit_error_help,
                               "Aggregate Double-bit ECC Errors:",
@@ -611,7 +685,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     
     /* Add buttons */
 
-    if ( dbit_error_available ) {
+    if (sbit_error_available && dbit_error_available) {
         ctk_ecc->clear_button = gtk_button_new_with_label("Clear ECC Errors");
         gtk_box_pack_end(GTK_BOX(hbox), ctk_ecc->clear_button, FALSE, FALSE, 0);
         ctk_config_set_tooltip(ctk_config, ctk_ecc->clear_button,
@@ -622,7 +696,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
                          (gpointer) ctk_ecc);
     }
 
-    if ( aggregate_dbit_error_available ) {
+    if (aggregate_sbit_error_available && aggregate_dbit_error_available) {
         ctk_ecc->clear_aggregate_button =
             gtk_button_new_with_label("Clear Aggregate ECC Errors");
         gtk_box_pack_end(GTK_BOX(hbox), ctk_ecc->clear_aggregate_button,
@@ -654,7 +728,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
 
     /* Register a timer callback to update Ecc status info */
     str = g_strdup_printf("ECC Settings (GPU %d)",
-                        NvCtrlGetTargetId(handle));
+                        NvCtrlGetTargetId(ctrl_target));
 
     ctk_config_add_timer(ctk_ecc->ctk_config,
                          DEFAULT_UPDATE_ECC_STATUS_INFO_TIME_INTERVAL,
@@ -687,11 +761,16 @@ GtkTextBuffer *ctk_ecc_create_help(GtkTextTagTable *table,
     ctk_help_heading(b, &i, "ECC");
     ctk_help_para(b, &i, "%s", __ecc_status_help);
 
-    if (ctk_ecc->dbit_error_available) {
+    if (ctk_ecc->sbit_error_available && ctk_ecc->dbit_error_available) {
+        ctk_help_heading(b, &i, "Single-bit ECC Errors");
+        ctk_help_para(b, &i, "%s", __sbit_error_help);
         ctk_help_heading(b, &i, "Double-bit ECC Errors");
         ctk_help_para(b, &i, "%s", __dbit_error_help);
     }
-    if (ctk_ecc->aggregate_dbit_error_available) {
+    if (ctk_ecc->aggregate_sbit_error_available &&
+        ctk_ecc->aggregate_dbit_error_available) {
+        ctk_help_heading(b, &i, "Aggregate Single-bit ECC Errors");
+        ctk_help_para(b, &i, "%s", __aggregate_sbit_error_help);
         ctk_help_heading(b, &i, "Aggregate Double-bit ECC Errors");
         ctk_help_para(b, &i, "%s", __aggregate_dbit_error_help);
     }
@@ -701,11 +780,12 @@ GtkTextBuffer *ctk_ecc_create_help(GtkTextTagTable *table,
     ctk_help_heading(b, &i, "Enable ECC");
     ctk_help_para(b, &i, "%s", __ecc_status_help);
 
-    if (ctk_ecc->dbit_error_available) {
+    if (ctk_ecc->sbit_error_available && ctk_ecc->dbit_error_available) {
         ctk_help_heading(b, &i, "Clear ECC Errors");
         ctk_help_para(b, &i, "%s", __clear_button_help);
     }
-    if (ctk_ecc->aggregate_dbit_error_available) {
+    if (ctk_ecc->aggregate_sbit_error_available &&
+        ctk_ecc->aggregate_dbit_error_available) {
         ctk_help_heading(b, &i, "Clear Aggregate ECC Errors");
         ctk_help_para(b, &i, "%s", __clear_aggregate_button_help);
     }
