@@ -25,22 +25,19 @@
 #include "ctkscale.h"
 #include "ctkhelp.h"
 #include "ctkthermal.h"
-#include "ctklicense.h"
 #include "ctkgauge.h"
 #include "ctkbanner.h"
 
 #define FRAME_PADDING 10
 #define DEFAULT_UPDATE_THERMAL_INFO_TIME_INTERVAL 1000
 
-static gboolean __license_accepted = FALSE;
-
 static gboolean update_thermal_info(gpointer);
 static gboolean update_cooler_info(gpointer);
 static void sync_gui_sensitivity(CtkThermal *ctk_thermal);
 static void sync_gui_to_modify_cooler_level(CtkThermal *ctk_thermal);
 static gboolean sync_gui_to_update_cooler_event(gpointer user_data);
 static void cooler_control_checkbox_toggled(GtkWidget *widget, gpointer user_data);
-static void cooler_operating_level_changed(GtkObject *object, gpointer arg1,
+static void cooler_operating_level_changed(GObject *object, CtrlEvent *event,
                                            gpointer user_data);
 static void apply_button_clicked(GtkWidget *widget, gpointer user_data);
 static void reset_button_clicked(GtkWidget *widget, gpointer user_data);
@@ -49,7 +46,6 @@ static void adjustment_value_changed(GtkAdjustment *adjustment,
 
 static void draw_sensor_gui(GtkWidget *vbox1, CtkThermal *ctk_thermal,
                             gboolean new_target_type, gint cur_sensor_idx,
-                            NvCtrlAttributeHandle *sensor_handle,
                             gint reading, gint lower, gint upper,
                             gint target, gint provider);
 static GtkWidget *pack_gauge(GtkWidget *hbox, gint lower, gint upper,
@@ -97,8 +93,11 @@ static const char * __enable_button_help =
 static const char * __fan_id_help =
 "This shows the GPU Fan's index.";
 
+static const char * __fan_rpm_help =
+"This shows the current GPU Fan Speed in rotations per minute (RPM).";
+
 static const char * __fan_speed_help =
-"This shows the current GPU Fan Speed.";
+"This shows the current GPU Fan Speed level as a percentage.";
 
 static const char * __fan_control_type_help =
 "Fan Type indicates if and how this fan may be controlled.  Possible "
@@ -155,7 +154,7 @@ GType ctk_thermal_get_type(void)
  */
 static gboolean update_cooler_info(gpointer user_data)
 {
-    int i, level, cooler_type, cooler_target;
+    int i, speed, level, cooler_type, cooler_target;
     gchar *tmp_str;
     CtkThermal *ctk_thermal;
     GtkWidget *table, *label, *eventbox;
@@ -190,18 +189,26 @@ static gboolean update_cooler_info(gpointer user_data)
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_id_help);
 
-    label = gtk_label_new("Speed (%)");
+    label = gtk_label_new("Speed (RPM)");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
     gtk_table_attach(GTK_TABLE(table), eventbox, 1, 2, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
+    ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_rpm_help);
+
+    label = gtk_label_new("Speed (%)");
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+    eventbox = gtk_event_box_new();
+    gtk_table_attach(GTK_TABLE(table), eventbox, 2, 3, 0, 1,
+                     GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
+    gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_speed_help);
 
     label = gtk_label_new("Control Type");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
-    gtk_table_attach(GTK_TABLE(table), eventbox, 2, 3, 0, 1,
+    gtk_table_attach(GTK_TABLE(table), eventbox, 3, 4, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox,
@@ -210,7 +217,7 @@ static gboolean update_cooler_info(gpointer user_data)
     label = gtk_label_new("Cooling Target");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
-    gtk_table_attach(GTK_TABLE(table), eventbox, 3, 4, 0, 1,
+    gtk_table_attach(GTK_TABLE(table), eventbox, 4, 5, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox,
@@ -220,7 +227,7 @@ static gboolean update_cooler_info(gpointer user_data)
     for (i = 0; i < ctk_thermal->cooler_count; i++) {
         row_idx = i+1;
 
-        gtk_table_resize(GTK_TABLE(table), row_idx+1, 4);
+        gtk_table_resize(GTK_TABLE(table), row_idx+1, 5);
 
         tmp_str = g_strdup_printf("%d", i);
         label = gtk_label_new(tmp_str);
@@ -229,7 +236,22 @@ static gboolean update_cooler_info(gpointer user_data)
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
-        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
+                                 NV_CTRL_THERMAL_COOLER_SPEED,
+                                 &speed);
+        if (ret == NvCtrlSuccess) {
+            tmp_str = g_strdup_printf("%d", speed);
+        }
+        else {
+            tmp_str = g_strdup_printf("Unsupported");
+        }
+        label = gtk_label_new(tmp_str);
+        gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+        gtk_table_attach(GTK_TABLE(table), label, 1, 2, row_idx, row_idx+1,
+                         GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
+        free(tmp_str);
+
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
                                  NV_CTRL_THERMAL_COOLER_LEVEL,
                                  &level);
         if (ret != NvCtrlSuccess) {
@@ -239,11 +261,11 @@ static gboolean update_cooler_info(gpointer user_data)
         tmp_str = g_strdup_printf("%d", level);
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 1, 2, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 2, 3, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
-        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
                                  NV_CTRL_THERMAL_COOLER_CONTROL_TYPE,
                                  &cooler_type);
         if (ret != NvCtrlSuccess) {
@@ -259,11 +281,11 @@ static gboolean update_cooler_info(gpointer user_data)
         }
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 2, 3, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 3, 4, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
-        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
                                  NV_CTRL_THERMAL_COOLER_TARGET,
                                  &cooler_target);
         if (ret != NvCtrlSuccess) {
@@ -288,7 +310,7 @@ static gboolean update_cooler_info(gpointer user_data)
         }
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 3, 4, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 4, 5, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
     }
@@ -308,17 +330,15 @@ static gboolean update_cooler_info(gpointer user_data)
 static gboolean update_thermal_info(gpointer user_data)
 {
     gint reading, ambient;
-    CtkThermal *ctk_thermal;
-    NvCtrlAttributeHandle *handle;
+    CtkThermal *ctk_thermal = CTK_THERMAL(user_data);
     gint ret, i, core;
     gchar *s;
 
-    ctk_thermal = CTK_THERMAL(user_data);
-
     if (!ctk_thermal->thermal_sensor_target_type_supported) {
-        handle = ctk_thermal->attribute_handle;
+        CtrlTarget *ctrl_target = ctk_thermal->ctrl_target;
 
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_CORE_TEMPERATURE, &core);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_CORE_TEMPERATURE,
+                                 &core);
         if (ret != NvCtrlSuccess) {
             /* thermal information no longer available */
             return FALSE;
@@ -332,7 +352,8 @@ static gboolean update_thermal_info(gpointer user_data)
         ctk_gauge_draw(CTK_GAUGE(ctk_thermal->core_gauge));
 
         if (ctk_thermal->ambient_label) {
-            ret = NvCtrlGetAttribute(handle, NV_CTRL_AMBIENT_TEMPERATURE,
+            ret = NvCtrlGetAttribute(ctrl_target,
+                                     NV_CTRL_AMBIENT_TEMPERATURE,
                                      &ambient);
             if (ret != NvCtrlSuccess) {
                 /* thermal information no longer available */
@@ -344,9 +365,10 @@ static gboolean update_thermal_info(gpointer user_data)
         }
     } else {
         for (i = 0; i < ctk_thermal->sensor_count; i++) {
-            handle = ctk_thermal->sensor_info[i].handle;
+            CtrlTarget *ctrl_target = ctk_thermal->sensor_info[i].ctrl_target;
 
-            ret = NvCtrlGetAttribute(handle, NV_CTRL_THERMAL_SENSOR_READING,
+            ret = NvCtrlGetAttribute(ctrl_target,
+                                     NV_CTRL_THERMAL_SENSOR_READING,
                                      &reading);
             /* querying THERMAL_SENSOR_READING failed: assume the temperature is 0 */
             if (ret != NvCtrlSuccess) {
@@ -384,6 +406,7 @@ static gboolean update_thermal_info(gpointer user_data)
  */
 static void cooler_control_state_update_gui(CtkThermal *ctk_thermal)
 {
+    CtrlTarget *ctrl_target = ctk_thermal->ctrl_target;
     ReturnStatus ret;
     int value;
     gboolean enabled;
@@ -394,7 +417,7 @@ static void cooler_control_state_update_gui(CtkThermal *ctk_thermal)
      * because the set might have failed.
      */
 
-    ret = NvCtrlGetAttribute(ctk_thermal->attribute_handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_GPU_COOLER_MANUAL_CONTROL,
                              &value);
     if (ret != NvCtrlSuccess) {
@@ -428,9 +451,9 @@ static void cooler_control_state_update_gui(CtkThermal *ctk_thermal)
 static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
 {
     CtkThermal *ctk_thermal = CTK_THERMAL(user_data);
+    CtrlTarget *ctrl_target = ctk_thermal->ctrl_target;
     gboolean enabled;
     int value;
-    gint result;
 
     /* Get enabled state */
 
@@ -438,42 +461,10 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
     value = (enabled==1) ? NV_CTRL_GPU_COOLER_MANUAL_CONTROL_TRUE :
         NV_CTRL_GPU_COOLER_MANUAL_CONTROL_FALSE;
 
-    /* Verify user knows the risks involved */
-
-    if ( enabled && !__license_accepted ) {
-
-        result =
-            ctk_license_run_dialog(CTK_LICENSE_DIALOG(ctk_thermal->enable_dialog));
-
-        switch (result) {
-        case GTK_RESPONSE_ACCEPT:
-            __license_accepted = TRUE;
-            break;
-
-        case GTK_RESPONSE_REJECT:
-        default:
-            /* Cancel */
-            g_signal_handlers_block_by_func(G_OBJECT(
-                                    ctk_thermal->enable_checkbox),
-                                    G_CALLBACK(cooler_control_state_toggled),
-                                    (gpointer) ctk_thermal);
-
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
-                                         FALSE);
-
-            g_signal_handlers_unblock_by_func(G_OBJECT(
-                                    ctk_thermal->enable_checkbox),
-                                    G_CALLBACK(cooler_control_state_toggled),
-                                    (gpointer) ctk_thermal);
-            return;
-        }
-    }
-
-
     /* Update the server */
 
-    NvCtrlSetAttribute(ctk_thermal->attribute_handle,
-                       NV_CTRL_GPU_COOLER_MANUAL_CONTROL, value);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_GPU_COOLER_MANUAL_CONTROL,
+                       value);
 
     /* Update the GUI */
 
@@ -489,8 +480,9 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
  * cooler control state.
  *
  */
-static void cooler_control_state_received(GtkObject *object,
-                                          gpointer arg1, gpointer user_data)
+static void cooler_control_state_received(GObject *object,
+                                          CtrlEvent *event,
+                                          gpointer user_data)
 {
     CtkThermal *ctk_thermal = CTK_THERMAL(user_data);
 
@@ -574,25 +566,26 @@ static void apply_button_clicked(GtkWidget *widget, gpointer user_data)
                 if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
                                        ctk_thermal->cooler_control[i].widget))) {
                     cooler_level =
-                        ctk_thermal->cooler_control[i].range.u.range.max;
+                        ctk_thermal->cooler_control[i].range.range.max;
                 } else { 
                     cooler_level = 
-                        ctk_thermal->cooler_control[i].range.u.range.min;
+                        ctk_thermal->cooler_control[i].range.range.min;
                 }
             }
 
-            ret = NvCtrlSetAttribute(ctk_thermal->cooler_control[i].handle,
-                                     NV_CTRL_THERMAL_COOLER_LEVEL,
-                                     cooler_level);
+            ret =
+                NvCtrlSetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
+                                   NV_CTRL_THERMAL_COOLER_LEVEL,
+                                   cooler_level);
 
             if ( ret != NvCtrlSuccess ) {
                 ctk_config_statusbar_message(ctk_thermal->ctk_config,
                                              "Failed to set new Fan Speed!");
                 return;
             }
+            ctk_thermal->cooler_control[i].changed = FALSE;
         }
     }
-    ctk_thermal->cooler_control[i].changed = FALSE;
     ctk_thermal->settings_changed = FALSE;
     ctk_thermal->enable_reset_button = TRUE;
     
@@ -629,7 +622,7 @@ static void reset_button_clicked(GtkWidget *widget, gpointer user_data)
     /* Set cooler related values to default */
 
     for (i = 0; i < ctk_thermal->cooler_count; i++) {
-        ret = NvCtrlSetAttribute(ctk_thermal->cooler_control[i].handle,
+        ret = NvCtrlSetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
                                  NV_CTRL_THERMAL_COOLER_LEVEL_SET_DEFAULT,
                                  NV_CTRL_THERMAL_COOLER_LEVEL_SET_DEFAULT);
         if ( ret != NvCtrlSuccess ) {
@@ -708,7 +701,7 @@ static void sync_gui_to_modify_cooler_level(CtkThermal *ctk_thermal)
 {
     GtkRange *gtk_range;
     GtkAdjustment *gtk_adjustment_fan;
-    NVCTRLAttributeValidValuesRec cooler_range;
+    CtrlAttributeValidValues cooler_range;
     gboolean can_access_cooler_level = TRUE;
     ReturnStatus ret;
     gint val, i, enabled;
@@ -718,7 +711,7 @@ static void sync_gui_to_modify_cooler_level(CtkThermal *ctk_thermal)
     for (i = 0; i < ctk_thermal->cooler_count; i++) {
         /* Obtain the current value and range of the fan speed */
         
-        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].ctrl_target,
                                  NV_CTRL_THERMAL_COOLER_LEVEL,
                                  &cooler_level);
         if ( ret != NvCtrlSuccess ) {
@@ -731,7 +724,7 @@ static void sync_gui_to_modify_cooler_level(CtkThermal *ctk_thermal)
             if ( ctk_thermal->cooler_control[i].adjustment ) {
 
                 ret = NvCtrlGetValidAttributeValues
-                    (ctk_thermal->cooler_control[i].handle,
+                    (ctk_thermal->cooler_control[i].ctrl_target,
                      NV_CTRL_THERMAL_COOLER_LEVEL,
                      &cooler_range);
                 if ( ret != NvCtrlSuccess ) {
@@ -748,8 +741,8 @@ static void sync_gui_to_modify_cooler_level(CtkThermal *ctk_thermal)
                                         G_CALLBACK(adjustment_value_changed),
                                         (gpointer) ctk_thermal);
                     gtk_range_set_range(gtk_range,
-                      ctk_thermal->cooler_control[i].range.u.range.min,
-                      ctk_thermal->cooler_control[i].range.u.range.max);
+                      ctk_thermal->cooler_control[i].range.range.min,
+                      ctk_thermal->cooler_control[i].range.range.max);
 
                     val = gtk_adjustment_get_value(gtk_adjustment_fan);
                     if (val != ctk_thermal->cooler_control[i].level) {
@@ -818,7 +811,8 @@ static gboolean sync_gui_to_update_cooler_event(gpointer user_data)
  * Callback function when another NV_CONTROL client changed cooler level.
  *
  */
-static void cooler_operating_level_changed(GtkObject *object, gpointer arg1,
+static void cooler_operating_level_changed(GObject *object,
+                                           CtrlEvent *event,
                                            gpointer user_data)
 {
     /* sync_gui_to_modify_cooler_level() to be called once when all other
@@ -942,7 +936,6 @@ static GtkWidget *pack_gauge(GtkWidget *hbox, gint lower, gint upper,
  */
 static void draw_sensor_gui(GtkWidget *vbox1, CtkThermal *ctk_thermal,
                             gboolean new_target_type, gint cur_sensor_idx,
-                            NvCtrlAttributeHandle *sensor_handle,
                             gint reading, gint lower, gint upper,
                             gint target, gint provider)
 {
@@ -1046,14 +1039,14 @@ static void draw_sensor_gui(GtkWidget *vbox1, CtkThermal *ctk_thermal,
 
 
 
-GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
+GtkWidget* ctk_thermal_new(CtrlTarget *ctrl_target,
                            CtkConfig *ctk_config,
                            CtkEvent *ctk_event)
 {
     GObject *object;
     CtkThermal *ctk_thermal;
-    CtrlHandles *h;
-    GtkObject *adjustment;
+    CtrlSystem *system;
+    GtkAdjustment *adjustment;
     GtkWidget *hbox = NULL, *hbox1, *hbox2, *table, *vbox;
     GtkWidget *frame, *banner, *label;
     GtkWidget *vbox1;
@@ -1064,10 +1057,10 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
     GtkWidget *alignment;
     ReturnStatus ret;
     ReturnStatus ret1;
-    NvCtrlAttributeHandle *cooler_handle;
-    NvCtrlAttributeHandle *sensor_handle;
-    NVCTRLAttributeValidValuesRec cooler_range;
-    NVCTRLAttributeValidValuesRec sensor_range;
+    CtrlTarget *cooler_target;
+    CtrlTarget *sensor_target;
+    CtrlAttributeValidValues cooler_range;
+    CtrlAttributeValidValues sensor_range;
     gint trigger, core, ambient;
     gint upper;
     gchar *s;
@@ -1087,7 +1080,8 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
 
     /* make sure we have a handle */
 
-    g_return_val_if_fail(handle != NULL, NULL);
+    g_return_val_if_fail((ctrl_target != NULL) &&
+                         (ctrl_target->h != NULL), NULL);
 
     /* 
      * Check for NV-CONTROL protocol version. 
@@ -1095,9 +1089,9 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
      * This used for backward compatibility between new nvidia-settings
      * and older X driver
      */ 
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
-    ret1 = NvCtrlGetAttribute(handle,
+    ret1 = NvCtrlGetAttribute(ctrl_target,
                               NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
 
     if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
@@ -1108,19 +1102,22 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
     if (!thermal_sensor_target_type_supported) {
         /* check if this screen supports thermal querying */
 
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_CORE_TEMPERATURE, &core);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_CORE_TEMPERATURE,
+                                 &core);
         if (ret != NvCtrlSuccess) {
             /* thermal information unavailable */
             return NULL;
         }
 
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_MAX_CORE_THRESHOLD, &upper);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_MAX_CORE_THRESHOLD,
+                                 &upper);
         if (ret != NvCtrlSuccess) {
             /* thermal information unavailable */
             return NULL;
         }
 
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_CORE_THRESHOLD, &trigger);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_CORE_THRESHOLD,
+                                 &trigger);
         if (ret != NvCtrlSuccess) {
             /* thermal information unavailable */
             return NULL;
@@ -1128,16 +1125,16 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
     }
     /* Query the list of sensors attached to this GPU */
 
-    ret = NvCtrlGetBinaryAttribute(handle, 0,
+    ret = NvCtrlGetBinaryAttribute(ctrl_target, 0,
                                    NV_CTRL_BINARY_DATA_THERMAL_SENSORS_USED_BY_GPU,
                                    (unsigned char **)(&pDataSensor), &len);
     if ( ret == NvCtrlSuccess ) {
         sensor_count = pDataSensor[0];
     }
-    
+
     /* Query the list of coolers attached to this GPU */
 
-    ret = NvCtrlGetBinaryAttribute(handle, 0,
+    ret = NvCtrlGetBinaryAttribute(ctrl_target, 0,
                                    NV_CTRL_BINARY_DATA_COOLERS_USED_BY_GPU,
                                    (unsigned char **)(&pDataCooler), &len);
     if ( ret == NvCtrlSuccess ) {
@@ -1147,24 +1144,24 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
     /* return if sensor and Fan information not available */
     if ((thermal_sensor_target_type_supported && !sensor_count) &&
         !cooler_count) {
-        XFree(pDataSensor);
-        XFree(pDataCooler);
+        free(pDataSensor);
+        free(pDataCooler);
         return NULL;
     }
 
     /* create the CtkThermal object */
 
     object = g_object_new(CTK_TYPE_THERMAL, NULL);
-    
+
     ctk_thermal = CTK_THERMAL(object);
-    ctk_thermal->attribute_handle = handle;
+    ctk_thermal->ctrl_target = ctrl_target;
     ctk_thermal->ctk_config = ctk_config;
     ctk_thermal->settings_changed = FALSE;
     ctk_thermal->show_fan_control_frame = TRUE;
     ctk_thermal->cooler_count = cooler_count;
     ctk_thermal->sensor_count = sensor_count;
     ctk_thermal->thermal_sensor_target_type_supported = thermal_sensor_target_type_supported;
-    
+
     /* set container properties for the CtkThermal widget */
 
     gtk_box_set_spacing(GTK_BOX(ctk_thermal), 10);
@@ -1176,7 +1173,7 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
 
     /* Check if we can control cooler state */
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_COOLER_MANUAL_CONTROL,
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GPU_COOLER_MANUAL_CONTROL,
                              &value);
     if ( ret != NvCtrlSuccess ) {
         ctk_thermal->show_fan_control_frame = FALSE;
@@ -1188,14 +1185,10 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
     can_access_cooler_level = TRUE;
     ctk_thermal->cooler_control_enabled = cooler_control_enabled;
     ctk_thermal->enable_reset_button = FALSE;
-    
-    if ( cooler_control_enabled ) {
-        __license_accepted = TRUE;
-    }
 
-    /* Retrieve CtrlHandles from ctk_config */
+    /* Retrieve CtrlSystem from ctk_config */
 
-    h = ctk_config->pCtrlHandles;
+    system = ctk_config->pCtrlSystem;
 
     /* Thermal Information */
 
@@ -1217,45 +1210,47 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
         
         if (ctk_thermal->sensor_count > 0) {
             ctk_thermal->sensor_info = (SensorInfoPtr)
-                malloc(ctk_thermal->sensor_count * sizeof(SensorInfoRec));
+                nvalloc(ctk_thermal->sensor_count * sizeof(SensorInfoRec));
         }
 
         for (j = 1; j <= ctk_thermal->sensor_count; j++) {
             gint reading, target, provider;
-            sensor_handle =
-                h->targets[THERMAL_SENSOR_TARGET].t[pDataSensor[j]].h;
 
-            if ( !sensor_handle ) {
+            sensor_target = NvCtrlGetTarget(system, THERMAL_SENSOR_TARGET,
+                                            pDataSensor[j]);
+            if (sensor_target == NULL) {
                 continue;
             }
 
-            ctk_thermal->sensor_info[cur_sensor_idx].handle = sensor_handle;
+            ctk_thermal->sensor_info[cur_sensor_idx].ctrl_target =
+                sensor_target;
 
             /* check if this screen supports thermal querying */
 
-            ret = NvCtrlGetAttribute(sensor_handle, NV_CTRL_THERMAL_SENSOR_READING,
+            ret = NvCtrlGetAttribute(sensor_target, NV_CTRL_THERMAL_SENSOR_READING,
                                      &reading);
             if (ret != NvCtrlSuccess) {
                 /* sensor information unavailable */
                 reading = 0;
             }
 
-            ret = NvCtrlGetValidAttributeValues(sensor_handle,
-                                                NV_CTRL_THERMAL_SENSOR_READING, 
+            ret = NvCtrlGetValidAttributeValues(sensor_target,
+                                                NV_CTRL_THERMAL_SENSOR_READING,
                                                 &sensor_range);
             if (ret != NvCtrlSuccess) {
                 /* sensor information unavailable */
-                sensor_range.u.range.min = sensor_range.u.range.max = 0;
+                sensor_range.range.min = sensor_range.range.max = 0;
             }
 
-            ret = NvCtrlGetAttribute(sensor_handle, NV_CTRL_THERMAL_SENSOR_TARGET,
+            ret = NvCtrlGetAttribute(sensor_target,
+                                     NV_CTRL_THERMAL_SENSOR_TARGET,
                                      &target);
             if (ret != NvCtrlSuccess) {
                 /* sensor information unavailable */
                 target = 0;
             }
 
-            ret = NvCtrlGetAttribute(sensor_handle, NV_CTRL_THERMAL_SENSOR_PROVIDER,
+            ret = NvCtrlGetAttribute(sensor_target, NV_CTRL_THERMAL_SENSOR_PROVIDER,
                                      &provider);
             if (ret != NvCtrlSuccess) {
                 /* sensor information unavailable */
@@ -1263,9 +1258,9 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
             }
             /* print sensor related information */
             draw_sensor_gui(vbox, ctk_thermal, thermal_sensor_target_type_supported,
-                            cur_sensor_idx, sensor_handle,
-                            reading, sensor_range.u.range.min,
-                            sensor_range.u.range.max, target, provider);
+                            cur_sensor_idx,
+                            reading, sensor_range.range.min,
+                            sensor_range.range.max, target, provider);
             cur_sensor_idx++;
         }
     } else {
@@ -1288,7 +1283,8 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
         eventbox = gtk_event_box_new();
         gtk_box_pack_start(GTK_BOX(hbox2), eventbox, FALSE, FALSE, 0);
 
-        entry = gtk_entry_new_with_max_length(5);
+        entry = gtk_entry_new();
+        gtk_entry_set_max_length(GTK_ENTRY(entry), 5);
         gtk_container_add(GTK_CONTAINER(eventbox), entry);
         gtk_widget_set_sensitive(entry, FALSE);
         gtk_entry_set_width_chars(GTK_ENTRY(entry), 5);
@@ -1327,7 +1323,8 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
 
         /* Ambient Temperature */
 
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_AMBIENT_TEMPERATURE, &ambient);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_AMBIENT_TEMPERATURE,
+                                 &ambient);
 
         if (ret == NvCtrlSuccess) {
             hbox2 = gtk_hbox_new(FALSE, 0);
@@ -1392,55 +1389,55 @@ sensor_end:
         calloc(ctk_thermal->cooler_count, sizeof(CoolerControlRec));
 
     for (j = 1; j <= ctk_thermal->cooler_count; j++) {
-        cooler_handle = h->targets[COOLER_TARGET].t[pDataCooler[j]].h;
-        
-        if ( !cooler_handle ) {    
+
+        cooler_target = NvCtrlGetTarget(system, COOLER_TARGET, pDataCooler[j]);
+        if (cooler_target == NULL) {
             continue;
         }
 
         /* Get current cooler level and range */
-        
-        ret = NvCtrlGetAttribute(cooler_handle, 
+
+        ret = NvCtrlGetAttribute(cooler_target,
                                  NV_CTRL_THERMAL_COOLER_LEVEL,
                                  &cooler_level);
         if ( ret != NvCtrlSuccess ) {
             can_access_cooler_level = FALSE;
         }
 
-        ret = NvCtrlGetValidAttributeValues(cooler_handle,
+        ret = NvCtrlGetValidAttributeValues(cooler_target,
                                             NV_CTRL_THERMAL_COOLER_LEVEL,
                                             &cooler_range);
         if ( ret != NvCtrlSuccess ) {
             can_access_cooler_level = FALSE;
         }
-        
+
         ctk_thermal->cooler_control[cur_cooler_idx].level = cooler_level; 
         ctk_thermal->cooler_control[cur_cooler_idx].range = cooler_range;
-        ctk_thermal->cooler_control[cur_cooler_idx].handle = cooler_handle;
-        
+        ctk_thermal->cooler_control[cur_cooler_idx].ctrl_target = cooler_target;
+
         /* Create the object for receiving NV-CONTROL events */
-        
+
         ctk_thermal->cooler_control[cur_cooler_idx].event =
-            CTK_EVENT(ctk_event_new(cooler_handle));
-        
+            CTK_EVENT(ctk_event_new(cooler_target));
+
         if ( can_access_cooler_level && ctk_thermal->show_fan_control_frame ) { 
             /* 
              * Get NV_CTRL_THERMAL_COOLER_CONTROL_TYPE to decide cooler
              * control widget should be slider or checkbox.
              */
-            
-            ret = NvCtrlGetAttribute(cooler_handle,
+
+            ret = NvCtrlGetAttribute(cooler_target,
                                      NV_CTRL_THERMAL_COOLER_CONTROL_TYPE,
                                      &cooler_control_type);
-            if ((ret == NvCtrlSuccess) && 
+            if ((ret == NvCtrlSuccess) &&
                 (cooler_control_type ==
                  NV_CTRL_THERMAL_COOLER_CONTROL_TYPE_VARIABLE)) {
 
                 adjustment =
-                    gtk_adjustment_new(cooler_level,
-                                       cooler_range.u.range.min,
-                                       cooler_range.u.range.max,
-                         1, 5, 0.0);
+                    GTK_ADJUSTMENT(gtk_adjustment_new(cooler_level,
+                                                      cooler_range.range.min,
+                                                      cooler_range.range.max,
+                                                      1, 5, 0.0));
                 name = g_strdup_printf("Fan %d Speed", cur_cooler_idx);
                 scale = ctk_scale_new(GTK_ADJUSTMENT(adjustment), name,
                                       ctk_config, G_TYPE_INT);
@@ -1476,11 +1473,6 @@ sensor_end:
     }
     
     if ( ctk_thermal->cooler_count && ctk_thermal->show_fan_control_frame ) {
-        /* Create the enable dialog */
-
-        ctk_thermal->enable_dialog = ctk_license_dialog_new(GTK_WIDGET(ctk_thermal),
-                                                            "Thermal Settings");
-
         /* Create the Enable Cooler control checkbox widget */
 
         ctk_thermal->enable_checkbox =
@@ -1576,12 +1568,12 @@ sensor_end:
     }
 
 end:
-    XFree(pDataSensor);
+    free(pDataSensor);
     pDataSensor = NULL;
-    
-    XFree(pDataCooler);
+
+    free(pDataCooler);
     pDataCooler = NULL;
-    
+
     /* sync GUI to current server settings */
     
     sync_gui_to_modify_cooler_level(ctk_thermal);
@@ -1590,8 +1582,8 @@ end:
     /* Register a timer callback to update the temperatures */
 
     s = g_strdup_printf("Thermal Monitor (GPU %d)",
-                        NvCtrlGetTargetId(handle));
-    
+                        NvCtrlGetTargetId(ctrl_target));
+
     ctk_config_add_timer(ctk_thermal->ctk_config,
                          DEFAULT_UPDATE_THERMAL_INFO_TIME_INTERVAL,
                          s,
@@ -1623,32 +1615,32 @@ GtkTextBuffer *ctk_thermal_create_help(GtkTextTagTable *table,
 
     if (!ctk_thermal->thermal_sensor_target_type_supported) {
         ctk_help_heading(b, &i, "Slowdown Threshold");
-        ctk_help_para(b, &i, __core_threshold_help);
+        ctk_help_para(b, &i, "%s", __core_threshold_help);
 
         ctk_help_heading(b, &i, "Core Temperature");
-        ctk_help_para(b, &i, __core_temp_help);
+        ctk_help_para(b, &i, "%s", __core_temp_help);
 
         if (ctk_thermal->ambient_label) {
             ctk_help_heading(b, &i, "Ambient Temperature");
-            ctk_help_para(b, &i, __ambient_temp_help);
+            ctk_help_para(b, &i, "%s", __ambient_temp_help);
         }
     } else {
         ctk_help_title(b, &i, "Thermal Sensor Information Help");
 
         ctk_help_heading(b, &i, "ID");
-        ctk_help_para(b, &i, __thermal_sensor_id_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_id_help);
 
         ctk_help_heading(b, &i, "Temperature");
-        ctk_help_para(b, &i, __thermal_sensor_reading_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_reading_help);
 
         ctk_help_heading(b, &i, "Target");
-        ctk_help_para(b, &i, __thermal_sensor_target_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_target_help);
         
         ctk_help_heading(b, &i, "Provider");
-        ctk_help_para(b, &i, __thermal_sensor_provider_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_provider_help);
     }
     ctk_help_heading(b, &i, "Level");
-    ctk_help_para(b, &i, __temp_level_help);
+    ctk_help_para(b, &i, "%s", __temp_level_help);
 
 next_help:
     /* if Fan not available skip online help */
@@ -1659,26 +1651,29 @@ next_help:
     ctk_help_title(b, &i, "GPU Fan Settings Help");
 
     ctk_help_heading(b, &i, "ID");
-    ctk_help_para(b, &i, __fan_id_help);
+    ctk_help_para(b, &i, "%s", __fan_id_help);
+
+    ctk_help_heading(b, &i, "Speed (RPM)");
+    ctk_help_para(b, &i,"%s", __fan_rpm_help);
 
     ctk_help_heading(b, &i, "Speed (%%)");
-    ctk_help_para(b, &i, __fan_speed_help);
+    ctk_help_para(b, &i, "%s", __fan_speed_help);
 
     ctk_help_heading(b, &i, "Type");
-    ctk_help_para(b, &i, __fan_control_type_help);
+    ctk_help_para(b, &i, "%s", __fan_control_type_help);
 
     ctk_help_heading(b, &i, "Cooling Target");
-    ctk_help_para(b, &i, __fan_cooling_target_help);
+    ctk_help_para(b, &i, "%s", __fan_cooling_target_help);
 
     ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-    ctk_help_para(b, &i, __enable_button_help);
+    ctk_help_para(b, &i, "%s", __enable_button_help);
 
     if ( ctk_thermal->show_fan_control_frame ) {
         ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-        ctk_help_para(b, &i, __apply_button_help);
+        ctk_help_para(b, &i, "%s", __apply_button_help);
 
         ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-        ctk_help_para(b, &i, __reset_button_help);
+        ctk_help_para(b, &i, "%s", __reset_button_help);
     }
 done:
     ctk_help_finish(b);
